///
/// Configure udev:
/// https://github.com/Grix/helios_dac/blob/master/docs/udev_rules_for_linux.md
///
mod redis_ctrl;
mod conf;
mod errors;
mod point;
mod modifier;

use helios_dac::{
    self,
    NativeHeliosDacController,
    NativeHeliosDac,
    DeviceStatus,
    Frame,
};
use std::sync::atomic::{AtomicBool, Ordering};
use std::sync::Arc;
use redis_ctrl::{RedisCtrl,Order};
use conf::Conf;
use errors::{LJError,LJResult};
use point::Point;
use modifier::{Transformers,Translate};

const CENTER : (f32,f32) = (2000.0, 2000.0);

pub fn main() {
    match run_all() {
	Ok(()) => {},
	Err(err) => {
	    println!("Error: {}", err);
	}
    }
}

fn run_all() -> LJResult<()> {
    let Some(filename) = std::env::args().nth(1) else {
	return Err(Box::new(LJError::ConfigFileMissing));
    };    
    let config = Conf::new(&filename)?;
    let mut rs = RedisCtrl::new(&config.redis_url)?;
    
    let running = Arc::new(AtomicBool::new(true));
    let r = running.clone();
    ctrlc::set_handler(move || {
        r.store(false, Ordering::SeqCst);
    })?;

    let mut device = get_helios_device()?;
    
    while running.load(Ordering::SeqCst) {
	let order = rs.get_order(config.laser_id)?;
	if order != Order::Draw {
	    println!("{:?}", order);
	}

	let frame = get_next_frame(&config, 1000,
				   &mut rs, order == Order::Black)?;
	
	while let Ok(DeviceStatus::NotReady) = device.status() {
	}
        device.write_frame(frame)?;
    }

    println!("Exiting, stoping device.");
    device.stop()?;
    Ok(())
}

fn get_helios_device() -> LJResult<NativeHeliosDac> {
    let controller = NativeHeliosDacController::new()?;
    let devices = controller.list_devices()?;
    let Some(device) = devices.into_iter().next() else {
	return Err(Box::new(LJError::HeliosDeviceMissing));
    };
    let device = device.open()?;
    Ok(device)
}

fn get_next_frame(
    config: &Conf,
    speed: u32,
    rs: &mut RedisCtrl,
    _black: bool
) -> LJResult<Frame> {
    
    let line = rs.get(&format!("/pl/{}/0", config.laser_id))?;
    let line: Vec<Point> = line.into_iter().map(| tpl | tpl.into()).collect();
    let line = Translate::new(CENTER.0, CENTER.1).apply(&line);
    
    let mut line2 = vec![];
    while line2.len() < 48 {
        for p in &line {
            line2.push(*p);
        }
    }

    println!("{:?}", line2);
    
    let line2 : Vec<helios_dac::Point> = line2.into_iter().map(| p | p.into()).collect();
    Ok(Frame::new(speed, line2))
}
